<?php
if (!defined('_PS_VERSION_'))
    exit;

class Treenation extends Module
{
    private $_html = '';
    private $_postErrors = array();

    public $treenation_id;
    public $treenation_url;

    public function __construct()
    {
        $this->name = 'treenation';
        $this->tab = 'front_office_features';
        $this->version = '1.1.0';
        $this->author = 'Tree-Nation';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->bootstrap = true;

        $config = Configuration::getMultiple(['TREENATION_ID','TREENATION_URL']);
        if (isset($config['TREENATION_ID'])) {
            $this->treenation_id = $config['TREENATION_ID'];
        }
        if (isset($config['TREENATION_URL'])) {
            $this->treenation_url = $config['TREENATION_URL'];
        }

        parent::__construct();

        $this->displayName = $this->l('Treenation');
        $this->description = $this->l('Webhook to process orders into Treenation website.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (empty($this->treenation_id)) {
            $this->warning = $this->l('Treenation ID must be provided to use this module.');
        }

    }

    public function install()
    {
        return parent::install()
            && $this->registerHook('actionPaymentConfirmation')
        ;
    }

    public function uninstall()
    {
        return Configuration::deleteByName('TREENATION_ID')
                && Configuration::deleteByName('TREENATION_URL')
                && parent::uninstall()
        ;
    }

    public function hookActionPaymentConfirmation(array $params)
    {
        $order = new Order((int)$params['id_order']);
        $customer = new Customer((int)$order->id_customer);
        $language = new Language((int)$customer->id_lang);
        $products = $params['cart']->getProducts();
        $webhookdata = [
            'cUrl' => $this->treenation_url ? : 'default', // field needed only for debug purposes
            'treenation_id' => $this->treenation_id,
            'order_id' => $params['id_order'],
            'email' => $customer->email,
            'name' => trim($customer->firstname . ' ' . $customer->lastname),
            'order_total' => $order->total_products_wt,
            'order_products' => [],
            'customer_locale' => $language->iso_code,
        ];
        foreach ($products as $product) {
            $webhookdata['order_products'][] = [
                'product_id' => $product['id_product'],
                'price' => $product['price_wt'],
                'quantity' => $product['quantity'],
            ];
        }
        PrestaShopLogger::addLog('payment hook was called. webhook data to send: '.json_encode($webhookdata));
        // send curl to configured tn url
        $result = $this->sendCurl($webhookdata);

        // log result
        if($result['success'] == true) {
            PrestaShopLogger::addLog('payment hook was sent. response data was: '.$result['success']);
        } else {
            PrestaShopLogger::addLog('payment hook was sent. cUrl ERROR: '.$result['error']);
        }

    }

    private function sendCurl($data)
    {
        $url = $this->treenation_url ? : 'https://tree-nation.com/prestashop-webhook';
        $fields = $data;

        //url-ify the data for the POST
        $fields_string = http_build_query($fields);
        //open connection
        $ch = curl_init();
        //set the url, number of POST vars, POST data
        curl_setopt($ch,CURLOPT_URL, $url);
        curl_setopt($ch,CURLOPT_POST, 1);
        curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, 1);

        //execute post
        $result = [
            'success' => curl_exec($ch),
            'error' => '',
        ];
        if ($result['success'] != true) $result['error'] = curl_error($ch);

        curl_close($ch);

        return $result;
    }

    public function getContent()
    {
        $this->_html = '';

        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        }

        $this->_html .= $this->_displayCheck();
        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    private function _postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('TREENATION_ID')) {
                $this->_postErrors[] = 'The "Treenation ID" field is required.';
            }
        }
    }

    private function _postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('TREENATION_ID', Tools::getValue('TREENATION_ID'));
            Configuration::updateValue('TREENATION_URL', Tools::getValue('TREENATION_URL'));
        }
        $this->_html .= $this->displayConfirmation('Settings updated');
    }

    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => 'Configuration',
                    'icon' => 'icon-tree'
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => 'Treenation ID',
                        'name' => 'TREENATION_ID',
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => 'Advanced parameters',
                        'name' => 'TREENATION_URL',
                        'required' => false,
                        'placeholder' => 'Leave it empty unless you‘ve been provided with this value',
                    ]
                ],
                'submit' => [
                    'title' => 'Save',
                ]
            ]
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
        );

        $this->fields_form = array();

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        return array(
            'TREENATION_ID' => Tools::getValue('TREENATION_ID', Configuration::get('TREENATION_ID')),
            'TREENATION_URL' => Tools::getValue('TREENATION_URL', Configuration::get('TREENATION_URL')),
        );
    }

    private function _displayCheck()
    {
        return $this->display(__FILE__, './views/templates/hook/infos.tpl');
    }

}
